@@ -39,6 +39,7 @@
 
 #include <unistd.h>
 #include <time.h>
+#include <math.h>
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
@@ -55,6 +56,7 @@
 #include "mainbox.h"
 #include "ical-code.h"
 #include "functions.h"
+#include "xfce_trayicon.h"
 
 #define MAX_APPT_LENGTH 4096
 #define LEN_BUFFER 1024
@@ -63,6 +65,9 @@
 
 #define XFICAL_STR_EXISTS(str) ((str != NULL) && (str[0] != 0))
 
+extern XfceTrayIcon *trayIcon;
+
+
 typedef struct
 {
     struct icaltimetype stime;
@@ -84,9 +89,8 @@ extern char *local_icaltimezone_location;
 extern gboolean local_icaltimezone_utc;
 
 static int lookback = 0;
-
 extern GList *alarm_list;
-                                                                                
+extern gint ical_weekstartday;
 
 /* Remember to keep this string table in sync with zones.tab
  * This is used only for translations purposes. It makes
@@ -477,6 +481,7 @@ const gchar *trans_timezone[] = {
     N_("Pacific/Wallis"),
     N_("Pacific/Yap"),
 };
+
 xfical_timezone_array xfical_get_timezones()
 {
     static xfical_timezone_array tz={0, NULL};
@@ -544,7 +549,7 @@ gboolean xfical_set_local_timezone(char *location)
     if XFICAL_STR_EXISTS(location) {
         local_icaltimezone_location = g_strdup(location);
         if (!local_icaltimezone_location) {
-            g_warning("Orage xfical_set_local_timezone: strdup memory exhausted");
+            g_warning("xfical_set_local_timezone: strdup memory exhausted");
             return (FALSE);
         }
         if (strcmp(location,"UTC") == 0) {
@@ -552,14 +557,14 @@ gboolean xfical_set_local_timezone(char *location)
             local_icaltimezone = utc_icaltimezone;
         }
         else if (strcmp(location,"floating") == 0) {
-            g_warning("Orage xfical_set_local_timezone: default timezone set to floating. Do not use timezones when setting appointments, it does not make sense without proper local timezone.");
+            g_warning("Default timezone set to floating. Do not use timezones when setting appointments, it does not make sense without proper local timezone.");
             return(TRUE); /* local_icaltimezone_location = NULL */
         }
         else
             local_icaltimezone = icaltimezone_get_builtin_timezone(location);
 
         if (!local_icaltimezone) {
-            g_warning("Orage xfical_set_local_timezone: builtin timezone %s not found"
+            g_warning("xfical_set_local_timezone: builtin timezone %s not found"
                     , location);
             return (FALSE);
         }
@@ -573,11 +578,11 @@ void xfical_add_timezone(icalcomponent *p_ical, icalset *p_fical, char *loc)
     icalcomponent *itimezone=NULL;
                                                                                 
     if (!loc) {
-        g_warning("Orage xfical_add_timezone: no location defined");
+        g_warning("xfical_add_timezone: no location defined");
         return;
     }
     else
-        g_message("adding timezone %s", loc);
+        g_message("Orage **: Adding timezone %s", loc);
 
     if (strcmp(loc,"UTC") == 0 
     ||  strcmp(loc,"floating") == 0) {
@@ -586,7 +591,7 @@ void xfical_add_timezone(icalcomponent *p_ical, icalset *p_fical, char *loc)
                                                                                 
     icaltz=icaltimezone_get_builtin_timezone(loc);
     if (icaltz==NULL) {
-        g_warning("Orage xfical_add_timezone: timezone not found %s", loc);
+        g_warning("xfical_add_timezone: timezone not found %s", loc);
         return;
     }
     itimezone=icaltimezone_get_component(icaltz);
@@ -596,7 +601,7 @@ void xfical_add_timezone(icalcomponent *p_ical, icalset *p_fical, char *loc)
         icalset_mark(p_fical);
     }
     else
-        g_warning("Orage xfical_add_timezone: timezone add failed %s", loc);
+        g_warning("xfical_add_timezone: timezone add failed %s", loc);
 }
 
 gboolean xfical_internal_file_open(icalcomponent **p_ical
@@ -607,7 +612,7 @@ gboolean xfical_internal_file_open(icalcomponent **p_ical
     gint cnt=0;
 
     if (*p_fical != NULL)
-        g_warning("Orage xfical_internal_file_open: file already open");
+        g_warning("xfical_internal_file_open: file already open");
     if ((*p_fical = icalset_new_file(file_icalpath)) == NULL) {
         g_error("xfical_internal_file_open: Could not open ical file (%s) %s\n"
                 , file_icalpath, icalerror_strerror(icalerrno));
@@ -629,7 +634,7 @@ gboolean xfical_internal_file_open(icalcomponent **p_ical
             *p_ical = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT
                    , icalproperty_new_version("2.0")
                    , icalproperty_new_prodid("-//Xfce//Orage//EN")
-                   , 0);
+                   , NULL);
             xfical_add_timezone(*p_ical, *p_fical, local_icaltimezone_location);
             icalset_add_component(*p_fical, *p_ical);
             /*
@@ -642,7 +647,7 @@ gboolean xfical_internal_file_open(icalcomponent **p_ical
         }
         else { /* VCALENDAR found */
             if (cnt > 1) {
-                g_warning("Orage xfical_internal_file_open: Too many top level components in calendar file %s", file_icalpath);
+                g_warning("xfical_internal_file_open: Too many top level components in calendar file %s", file_icalpath);
             }
         }
     }
@@ -667,7 +672,7 @@ gboolean xfical_archive_open (void)
 void xfical_file_close(void)
 {
     if (fical == NULL)
-        g_warning("Orage xfical_file_close: fical is NULL");
+        g_warning("xfical_file_close: fical is NULL");
     icalset_free(fical);
     fical = NULL;
 }
@@ -678,7 +683,7 @@ void xfical_archive_close(void)
         return;
 
     if (afical == NULL)
-        g_warning("Orage xfical_file_close: afical is NULL");
+        g_warning("xfical_file_close: afical is NULL");
     icalset_free(afical);
     afical = NULL;
 }
@@ -723,7 +728,7 @@ struct icaltimetype convert_to_timezone(struct icaltimetype t, icalproperty *p)
          tz_loc = (char *) icalparameter_get_tzid(itime_tz);
          l_icaltimezone = icaltimezone_get_builtin_timezone(tz_loc);
          if (!l_icaltimezone) {
-            g_warning("Orage convert_to_timezone: builtin timezone %s not found, conversion failed.", tz_loc);
+            g_warning("convert_to_timezone: builtin timezone %s not found, conversion failed.", tz_loc);
         }
         tz = icaltime_convert_to_zone(t, l_icaltimezone);
     }
@@ -748,6 +753,8 @@ xfical_period get_period(icalcomponent *c_event)
 {
     icalproperty *p;
     xfical_period per;
+    struct icaldurationtype duration_tmp;
+    gint dur_int;
 
     /* Exactly one start time must be there */
     p = icalcomponent_get_first_property(c_event, ICAL_DTSTART_PROPERTY);
@@ -756,7 +763,7 @@ xfical_period get_period(icalcomponent *c_event)
         per.stime = convert_to_local_timezone(per.stime, p);
     }
     else {
-        g_warning("Orage get_period: start time not found (%s)"
+        g_warning("get_period: start time not found (%s)"
                 , icalcomponent_get_uid(c_event));
         per.stime = icaltime_null_time();
     } 
@@ -767,6 +774,16 @@ xfical_period get_period(icalcomponent *c_event)
         per.etime = icalproperty_get_dtend(p);
         per.etime = convert_to_local_timezone(per.etime, p);
         per.duration = icaltime_subtract(per.etime, per.stime);
+        if (icaltime_is_date(per.stime)
+            && icaldurationtype_as_int(per.duration) != 0) {
+        /* need to subtract 1 day.
+         * read explanation from appt_add_internal: appt->endtime processing */
+            duration_tmp = icaldurationtype_from_int(60*60*24);
+            dur_int = icaldurationtype_as_int(per.duration);
+            dur_int -= icaldurationtype_as_int(duration_tmp);
+            per.duration = icaldurationtype_from_int(dur_int);
+            per.etime = icaltime_add(per.stime, per.duration);
+        }
     }
     else {
         p = icalcomponent_get_first_property(c_event, ICAL_DURATION_PROPERTY);
@@ -775,7 +792,8 @@ xfical_period get_period(icalcomponent *c_event)
             per.etime = icaltime_add(per.stime, per.duration);
         }
         else {
-            g_message("Orage get_period: end time/duration not found");
+            g_warning("get_period: end time/duration not found (%s)"
+                , icalcomponent_get_uid(c_event));
             per.etime = per.stime;
             per.duration = icaldurationtype_null_duration();
         } 
@@ -882,7 +900,7 @@ struct icaltimetype convert_to_zone(struct icaltimetype t, gchar *tz)
         else {
             l_icaltimezone = icaltimezone_get_builtin_timezone(tz);
             if (!l_icaltimezone)
-                g_warning("Orage convert_to_zone: builtin timezone %s not found, conversion failed.", tz);
+                g_warning("convert_to_zone: builtin timezone %s not found, conversion failed.", tz);
             else
                 wtime = icaltime_convert_to_zone(t, l_icaltimezone);
         }
@@ -902,7 +920,7 @@ int xfical_compare_times(appt_data *appt)
 
     if (appt->use_duration) {
         if (! XFICAL_STR_EXISTS(appt->starttime)) {
-            g_warning("Orage xfical_compare_times: null start time");
+            g_warning("xfical_compare_times: null start time");
             return(0); /* should be error ! */
         }
         stime = icaltime_from_string(appt->starttime);
@@ -931,7 +949,7 @@ int xfical_compare_times(appt_data *appt)
             return (icaltime_compare(stime, etime));
         }
         else {
-            g_warning("Orage xfical_compare_times: null time %s %s"
+            g_warning("xfical_compare_times: null time %s %s"
                     , appt->starttime, appt->endtime);
             return(0); /* should be error ! */
         }
@@ -944,11 +962,14 @@ int xfical_compare_times(appt_data *appt)
   */
 appt_data *xfical_appt_alloc()
 {
-    appt_data *temp;
+    appt_data *appt;
+    int i;
 
-    temp = g_new0(appt_data, 1);
-    temp->availability = 1;
-    return(temp);
+    appt = g_new0(appt_data, 1);
+    appt->availability = 1;
+    for (i=0; i <= 6; i++)
+        appt->recur_byday[i] = TRUE;
+    return(appt);
 }
 
 void alarm_free(gpointer galarm, gpointer dummy)
@@ -985,6 +1006,7 @@ void alarm_add(icalproperty_status action
              , struct icaltimetype alarm_time, struct icaltimetype event_time)
 {
     alarm_struct *new_alarm;
+
     new_alarm = g_new(alarm_struct, 1);
     new_alarm->uid = g_string_new(uid);
     new_alarm->title = g_string_new(title);
@@ -1002,6 +1024,7 @@ void alarm_add(icalproperty_status action
         new_alarm->audio = TRUE;
     else
         new_alarm->audio = FALSE;
+
     alarm_list = g_list_append(alarm_list, new_alarm);
 }
 
@@ -1097,7 +1120,7 @@ void xfical_alarm_build_list_internal(gboolean first_list_today)
                         repeat_delay = icaldurationtype_as_int(duration);
                         break;
                     default:
-                        g_warning("Orage ical-code: Unknown property (%s) in Alarm",
+                        g_warning("ical-code: Unknown property (%s) in Alarm",
                             (char *)icalproperty_get_property_name(p));
                         break;
                 } 
@@ -1131,7 +1154,7 @@ void xfical_alarm_build_list_internal(gboolean first_list_today)
                 ri = icalrecur_iterator_new(rrule, alarm_time);
                 for (next_date = icalrecur_iterator_next(ri);
                     (!icaltime_is_null_time(next_date))
-                    && (local_compare_date_only(cur_time, next_date) > 0) ;
+                    && (local_compare(cur_time, next_date) > 0) ;
                     next_date = icalrecur_iterator_next(ri)) {
                     cnt_repeat++;
                 }
@@ -1146,8 +1169,9 @@ void xfical_alarm_build_list_internal(gboolean first_list_today)
         } /* trg_found */
     }  /* EVENTS */
     alarm_list = g_list_sort(alarm_list, alarm_order);
-    g_message("Orage build alarm list: Processed %d events. Found %d alarms of which %d are active. (Searched %d recurring alarms.)"
-            , cnt_event, cnt_alarm, cnt_act_alarm, cnt_repeat);
+    if (first_list_today)
+        g_message("Orage **: Build alarm list: Processed %d events.\n\tFound %d alarms of which %d are active. (Searched %d recurring alarms.)"
+                , cnt_event, cnt_alarm, cnt_act_alarm, cnt_repeat);
 }
 
 void xfical_alarm_build_list(gboolean first_list_today)
@@ -1168,6 +1192,23 @@ gboolean xfical_alarm_passed(char *alarm_stime)
     return(FALSE);
  }
 
+gboolean xfical_duration(char *alarm_stime, int *days, int *hours, int *mins)
+{
+    struct icaltimetype alarm_time, cur_time;
+    struct icaldurationtype duration;
+
+    duration = icaldurationtype_null_duration();
+    cur_time = ical_get_current_local_time();
+    alarm_time = icaltime_from_string(alarm_stime);
+    duration = icaltime_subtract(alarm_time, cur_time);
+    if (icaldurationtype_is_bad_duration(duration) || duration.is_neg)
+        return(FALSE);
+    *days = duration.weeks*7 + duration.days;
+    *hours = duration.hours;
+    *mins = duration.minutes;
+    return(TRUE);
+ }
+
 void appt_add_alarm_internal(appt_data *appt, icalcomponent *ievent)
 {
     icalcomponent *ialarm;
@@ -1182,7 +1223,7 @@ void appt_add_alarm_internal(appt_data *appt, icalcomponent *ievent)
     ialarm = icalcomponent_vanew(ICAL_VALARM_COMPONENT
         , icalproperty_new_action(ICAL_ACTION_DISPLAY)
         , icalproperty_new_trigger(trg)
-        , 0);
+        , NULL);
     if XFICAL_STR_EXISTS(appt->note)
         icalcomponent_add_property(ialarm
             , icalproperty_new_description(appt->note));
@@ -1198,7 +1239,7 @@ void appt_add_alarm_internal(appt_data *appt, icalcomponent *ievent)
         ialarm = icalcomponent_vanew(ICAL_VALARM_COMPONENT
             , icalproperty_new_action(ICAL_ACTION_AUDIO)
             , icalproperty_new_trigger(trg)
-            , 0);
+            , NULL);
         attach = icalattach_new_from_url(appt->sound);
         icalcomponent_add_property(ialarm
             , icalproperty_new_attach(attach));
@@ -1220,9 +1261,13 @@ char *appt_add_internal(appt_data *appt, gboolean add, char *uid
     struct icaltimetype dtstamp, create_time, wtime;
     static gchar xf_uid[1001];
     gchar xf_host[501];
-    gchar recur_str[101], *recur_p;
+    gchar recur_str[1001], *recur_p, *recur_p2;
     struct icalrecurrencetype rrule;
     struct icaldurationtype duration;
+    int i, cnt;
+    gchar *byday_a[] = {"MO", "TU", "WE", "TH", "FR", "SA", "SU"};
+    icaltimezone *l_icaltimezone = NULL;
+    const char *text;
 
     dtstamp = icaltime_current_time_with_zone(utc_icaltimezone);
     if (add) {
@@ -1246,7 +1291,7 @@ char *appt_add_internal(appt_data *appt, gboolean add, char *uid
            , icalproperty_new_class(ICAL_CLASS_PUBLIC)
            , icalproperty_new_dtstamp(dtstamp)
            , icalproperty_new_created(create_time)
-           , 0);
+           , NULL);
 
     if XFICAL_STR_EXISTS(appt->title)
         icalcomponent_add_property(ievent
@@ -1270,7 +1315,11 @@ char *appt_add_internal(appt_data *appt, gboolean add, char *uid
 
     if XFICAL_STR_EXISTS(appt->starttime) {
         wtime=icaltime_from_string(appt->starttime);
-        if XFICAL_STR_EXISTS(appt->start_tz_loc) {
+        if (appt->allDay) { /* date */
+            icalcomponent_add_property(ievent
+                    , icalproperty_new_dtstart(wtime));
+        }
+        else if XFICAL_STR_EXISTS(appt->start_tz_loc) {
             if (strcmp(appt->start_tz_loc, "UTC") == 0) {
                 wtime=icaltime_convert_to_zone(wtime, utc_icaltimezone);
                 icalcomponent_add_property(ievent
@@ -1299,7 +1348,15 @@ char *appt_add_internal(appt_data *appt, gboolean add, char *uid
     }
     else if XFICAL_STR_EXISTS(appt->endtime) {
         wtime=icaltime_from_string(appt->endtime);
-        if XFICAL_STR_EXISTS(appt->end_tz_loc) {
+        if (appt->allDay) {
+            /* need to add 1 day. For example:
+             * DTSTART=20070221 & DTEND=20070223
+             * means only 21 and 22 February */
+            duration = icaldurationtype_from_int(60*60*24);
+            wtime = icaltime_add(wtime, duration);
+            icalcomponent_add_property(ievent, icalproperty_new_dtend(wtime));
+        }
+        else if XFICAL_STR_EXISTS(appt->end_tz_loc) {
             if (strcmp(appt->end_tz_loc, "UTC") == 0) {
                 wtime=icaltime_convert_to_zone(wtime, utc_icaltimezone);
                 icalcomponent_add_property(ievent
@@ -1337,14 +1394,68 @@ char *appt_add_internal(appt_data *appt, gboolean add, char *uid
                 recur_p = g_stpcpy(recur_p, "YEARLY");
                 break;
             default:
-                g_warning("Orage appt_add_internal: Unsupported freq");
+                g_warning("appt_add_internal: Unsupported freq");
                 icalrecurrencetype_clear(&rrule);
         }
+        if (appt->interval > 1) { /* not default, need to insert it */
+            recur_p += g_sprintf(recur_p, ";INTERVAL=%d", appt->interval);
+        }
         if (appt->recur_limit == 1) {
-            g_sprintf(recur_p, ";COUNT=%d", appt->recur_count);
+            recur_p += g_sprintf(recur_p, ";COUNT=%d", appt->recur_count);
         }
         else if (appt->recur_limit == 2) { /* needs to be in UTC */
-            g_sprintf(recur_p, ";UNTIL=%sZ", appt->recur_until);
+/* BUG 2937: convert recur_until to utc from start time timezone */
+            wtime = icaltime_from_string(appt->recur_until);
+            if XFICAL_STR_EXISTS(appt->start_tz_loc) {
+            /* Null == floating => no special action needed */
+                if (strcmp(appt->start_tz_loc, "floating") == 0) {
+                    wtime = icaltime_convert_to_zone(wtime
+                            , local_icaltimezone);
+                }
+                else if (strcmp(appt->start_tz_loc, "UTC") != 0) {
+                /* FIXME: add this vtimezone to vcalendar if it is not there */
+                    l_icaltimezone = icaltimezone_get_builtin_timezone(
+                            appt->start_tz_loc);
+                    wtime = icaltime_convert_to_zone(wtime, l_icaltimezone);
+                }
+            }
+            else
+                wtime = icaltime_convert_to_zone(wtime, local_icaltimezone);
+            wtime = icaltime_convert_to_zone(wtime, utc_icaltimezone);
+            text  = icaltime_as_ical_string(wtime);
+            recur_p += g_sprintf(recur_p, ";UNTIL=%s", text);
+        }
+        recur_p2 = recur_p; /* store current pointer */
+        for (i = 0, cnt = 0; i <= 6; i++) {
+            if (appt->recur_byday[i]) {
+                if (cnt == 0) /* first day found */
+                    recur_p = g_stpcpy(recur_p, ";BYDAY=");
+                else /* continue the list */
+                    recur_p = g_stpcpy(recur_p, ",");
+                if ((appt->freq == XFICAL_FREQ_MONTHLY
+                        || appt->freq == XFICAL_FREQ_YEARLY)
+                    && (appt->recur_byday_cnt[i])) /* number defined */
+                        recur_p += g_sprintf(recur_p, "%d"
+                                , appt->recur_byday_cnt[i]);
+                recur_p = g_stpcpy(recur_p, byday_a[i]);
+                cnt++;
+            }
+        }
+        if (cnt == 7) { /* all days defined... */
+            *recur_p2 = *recur_p; /* ...reset to null... */
+            recur_p = recur_p2;   /* ...no need for BYDAY then */
+        }
+        else if (appt->interval > 1 && appt->freq == XFICAL_FREQ_WEEKLY) {
+            /* we have BYDAY rule, let's check week starting date:
+             * WKST has meaning only in two cases:
+             * 1) WEEKLY rule && interval > 1 && BYDAY rule is in use 
+             * 2) YEARLY rule && BYWEEKNO rule is in use 
+             * BUT Orage is not using BYWEEKNO rule, so we only check 1)
+             * Monday is default, so we can skip that, too
+             * */
+            if (ical_weekstartday)
+                recur_p += g_sprintf(recur_p, ";WKST=%s"
+                        , byday_a[ical_weekstartday]);
         }
         rrule = icalrecurrencetype_from_string(recur_str);
         icalcomponent_add_property(ievent, icalproperty_new_rrule(rrule));
@@ -1392,7 +1503,7 @@ void ical_appt_get_alarm_internal(icalcomponent *c,  appt_data *appt)
                                             * -1;
                     }
                     else
-                        g_warning("Orage ical_appt_get_alarm_internal: Can not process time triggers");
+                        g_warning("ical_appt_get_alarm_internal: Can not process time triggers");
                     break;
                 case ICAL_ATTACH_PROPERTY:
                     attach = icalproperty_get_attach(p);
@@ -1410,7 +1521,7 @@ void ical_appt_get_alarm_internal(icalcomponent *c,  appt_data *appt)
                 /* no actions defined */
                     break;
                 default:
-                    g_warning("Orage ical_appt_get_alarm_internal: unknown property %s", (char *)icalproperty_get_property_name(p));
+                    g_warning("ical_appt_get_alarm_internal: unknown property %s", (char *)icalproperty_get_property_name(p));
                     break;
             }
         }
@@ -1439,13 +1550,14 @@ appt_data *xfical_appt_get_internal(char *ical_uid)
     icalparameter *itime_tz;
     icalproperty_transp xf_transp;
     struct icalrecurrencetype rrule;
-    struct icaldurationtype duration;
+    struct icaldurationtype duration, duration_tmp;
+    int i, cnt, day;
 
     for (c = icalcomponent_get_first_component(ical, ICAL_VEVENT_COMPONENT); 
          c != 0 && !key_found;
          c = icalcomponent_get_next_component(ical, ICAL_VEVENT_COMPONENT)) {
         text = icalcomponent_get_uid(c);
-        if (strcmp(text, ical_uid) == 0) {
+        if (strcmp(text, ical_uid) == 0) { /* we found our uid (=event) */
         /*********** Defaults ***********/
             stime = icaltime_null_time();
             sltime = icaltime_null_time();
@@ -1470,9 +1582,14 @@ appt_data *xfical_appt_get_internal(char *ical_uid)
             appt.starttimecur[0] = '\0';
             appt.endtimecur[0] = '\0';
             appt.freq = XFICAL_FREQ_NONE;
+            appt.interval = 1;
             appt.recur_limit = 0;
             appt.recur_count = 0;
             appt.recur_until[0] = '\0';
+            for (i=0; i <= 6; i++) {
+                appt.recur_byday[i] = TRUE;
+                appt.recur_byday_cnt[i] = 0;
+            }
         /*********** Properties ***********/
             for (p = icalcomponent_get_first_property(c, ICAL_ANY_PROPERTY);
                  p != 0;
@@ -1575,6 +1692,54 @@ appt_data *xfical_appt_get_internal(char *ical_uid)
                             text  = icaltime_as_ical_string(rrule.until);
                             g_strlcpy(appt.recur_until, text, 17);
                         }
+                        if (rrule.by_day[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                            for (i=0; i <= 6; i++)
+                                appt.recur_byday[i] = FALSE;
+                            for (i=0; i <= 6 && 
+                                 rrule.by_day[i] != ICAL_RECURRENCE_ARRAY_MAX; 
+                                 i++) {
+                                cnt = (int)floor((double)(rrule.by_day[i]/8));
+                                day = abs(rrule.by_day[i]-8*cnt);
+                                switch (day) {
+                                    case ICAL_MONDAY_WEEKDAY:
+                                        appt.recur_byday[0] = TRUE;
+                                        appt.recur_byday_cnt[0] = cnt;
+                                        break;
+                                    case ICAL_TUESDAY_WEEKDAY:
+                                        appt.recur_byday[1] = TRUE;
+                                        appt.recur_byday_cnt[1] = cnt;
+                                        break;
+                                    case ICAL_WEDNESDAY_WEEKDAY:
+                                        appt.recur_byday[2] = TRUE;
+                                        appt.recur_byday_cnt[2] = cnt;
+                                        break;
+                                    case ICAL_THURSDAY_WEEKDAY:
+                                        appt.recur_byday[3] = TRUE;
+                                        appt.recur_byday_cnt[3] = cnt;
+                                        break;
+                                    case ICAL_FRIDAY_WEEKDAY:
+                                        appt.recur_byday[4] = TRUE;
+                                        appt.recur_byday_cnt[4] = cnt;
+                                        break;
+                                    case ICAL_SATURDAY_WEEKDAY:
+                                        appt.recur_byday[5] = TRUE;
+                                        appt.recur_byday_cnt[5] = cnt;
+                                        break;
+                                    case ICAL_SUNDAY_WEEKDAY:
+                                        appt.recur_byday[6] = TRUE;
+                                        appt.recur_byday_cnt[6] = cnt;
+                                        break;
+                                    case ICAL_NO_WEEKDAY:
+                                        break;
+                                    default:
+                                        g_warning("xfical_appt_get_internal: unknown weekday %s: %d/%d (%x)"
+                                                , ical_uid, rrule.by_day[i]
+                                                , i, rrule.by_day[i]);
+                                        break;
+                                }
+                            }
+                        }
+                        appt.interval = rrule.interval;
                         break;
                     case ICAL_CATEGORIES_PROPERTY:
                     case ICAL_CLASS_PROPERTY:
@@ -1582,7 +1747,8 @@ appt_data *xfical_appt_get_internal(char *ical_uid)
                     case ICAL_CREATED_PROPERTY:
                         break;
                     default:
-                        g_warning("Orage xfical_appt_get: unknown property %s", (char *)icalproperty_get_property_name(p));
+                        g_warning("xfical_appt_get_internal: unknown property %s"
+                                , (char *)icalproperty_get_property_name(p));
                         break;
                 }
             }
@@ -1590,19 +1756,30 @@ appt_data *xfical_appt_get_internal(char *ical_uid)
         }
     } 
 
+    if (key_found) {
     /* need to set missing endtime or duration */
-    if (appt.use_duration) { 
-        etime = icaltime_add(stime, duration);
-        text  = icaltime_as_ical_string(etime);
-        g_strlcpy(appt.endtime, text, 17);
-        appt.end_tz_loc = appt.start_tz_loc;
-    }
-    else {
-        duration = icaltime_subtract(eltime, sltime);
-        appt.duration = icaldurationtype_as_int(duration);
-    }
-    if (key_found)
+        if (appt.use_duration) { 
+            etime = icaltime_add(stime, duration);
+            text  = icaltime_as_ical_string(etime);
+            g_strlcpy(appt.endtime, text, 17);
+            appt.end_tz_loc = appt.start_tz_loc;
+        }
+        else {
+            duration = icaltime_subtract(eltime, sltime);
+            appt.duration = icaldurationtype_as_int(duration);
+            if (appt.allDay && appt.duration) {
+        /* need to subtract 1 day.
+         * read explanation from appt_add_internal: appt->endtime processing */
+                duration_tmp = icaldurationtype_from_int(60*60*24);
+                appt.duration -= icaldurationtype_as_int(duration_tmp);
+                duration = icaldurationtype_from_int(appt.duration);
+                etime = icaltime_add(stime, duration);
+                text  = icaltime_as_ical_string(etime);
+                g_strlcpy(appt.endtime, text, 17);
+            }
+        }
         return(&appt);
+    }
     else
         return(NULL);
 }
@@ -1659,7 +1836,7 @@ gboolean xfical_appt_mod(char *ical_uid, appt_data *app)
     struct icaltimetype create_time = icaltime_null_time();
 
     if (ical_uid == NULL) {
-        g_warning("Orage xfical_appt_mod: Got NULL uid. doing nothing");
+        g_warning("xfical_appt_mod: Got NULL uid. doing nothing");
         return(FALSE);
     }
     for (c = icalcomponent_get_first_component(ical, ICAL_VEVENT_COMPONENT); 
@@ -1674,7 +1851,7 @@ gboolean xfical_appt_mod(char *ical_uid, appt_data *app)
         }
     } 
     if (!key_found) {
-        g_warning("Orage xfical_appt_mod: uid not found. doing nothing");
+        g_warning("xfical_appt_mod: uid not found. doing nothing");
         return(FALSE);
     }
 
@@ -1692,7 +1869,7 @@ gboolean xfical_appt_del(char *ical_uid)
     char *uid;
 
     if (ical_uid == NULL) {
-        g_warning("Orage xfical_appt_del: Got NULL uid. doing nothing");
+        g_warning("xfical_appt_del: Got NULL uid. doing nothing");
         return(FALSE);
      }
     for (c = icalcomponent_get_first_component(ical, ICAL_VEVENT_COMPONENT); 
@@ -1706,7 +1883,7 @@ gboolean xfical_appt_del(char *ical_uid)
             return(TRUE);
         }
     } 
-    g_warning("Orage xfical_appt_del: uid not found. doing nothing");
+    g_warning("xfical_appt_del: uid not found. doing nothing");
     return(FALSE);
 }
 
@@ -1726,7 +1903,6 @@ appt_data *xfical_appt_get_next_on_day(char *a_day, gboolean first, gint days)
     struct icaltimetype 
               asdate, aedate    /* period to check */
             , nsdate, nedate;   /* repeating event occurrency start and end */
-    struct icaldurationtype aduration;
     xfical_period per; /* event start and end times with duration */
     icalcomponent *c=NULL;
     icalproperty *p = NULL;
@@ -1740,13 +1916,9 @@ appt_data *xfical_appt_get_next_on_day(char *a_day, gboolean first, gint days)
 
     /* setup period to test */
     asdate = icaltime_from_string(a_day);
-    if (days) { /* more than one day to check */
-        aduration = icaldurationtype_null_duration();
-        aduration.days = days;
-        aedate = icaltime_add(asdate, aduration);
-    }
-    else /* only one day */
-        aedate = asdate;
+    aedate = asdate;
+    if (days)  /* more than one day to check */
+        icaltime_adjust(&aedate, days, 0, 0, 0);
 
     if (first)
         ci = icalcomponent_begin_component(ical, ICAL_VEVENT_COMPONENT);
@@ -1828,7 +2000,9 @@ void xfical_mark_calendar(GtkCalendar *gtkcal, int year, int month)
             else
                 end_day = 31;
             for (day_cnt = start_day; day_cnt <= end_day; day_cnt++)
+            {
                 gtk_calendar_mark_day(gtkcal, day_cnt);
+            }
         }
         if ((p = icalcomponent_get_first_property(c
                 , ICAL_RRULE_PROPERTY)) != 0) { /* check recurring EVENTs */
@@ -1842,8 +2016,8 @@ void xfical_mark_calendar(GtkCalendar *gtkcal, int year, int month)
                  nsdate = icalrecur_iterator_next(ri),
                     nedate = icaltime_add(nsdate, per.duration)) {
                 if ((nsdate.year*12+nsdate.month) <= (year*12+month)
-                        && (year*12+month) <= (nedate.year*12+nedate.month)) {
-                        /* event is in our year+month = visible in calendar */
+                    && (year*12+month) <= (nedate.year*12+nedate.month)) {
+                    /* event is in our year+month = visible in calendar */
                     if (nsdate.year == year && nsdate.month == month)
                         start_day = nsdate.day;
                     else
@@ -1853,7 +2027,9 @@ void xfical_mark_calendar(GtkCalendar *gtkcal, int year, int month)
                     else
                         end_day = 31;
                     for (day_cnt = start_day; day_cnt <= end_day; day_cnt++)
+                    {
                         gtk_calendar_mark_day(gtkcal, day_cnt);
+                    }
                 }
             }
             icalrecur_iterator_free(ri);
@@ -1933,7 +2109,7 @@ void xfical_icalcomponent_archive_recurrent(icalcomponent *e
          stz_loc = (char *) icalparameter_get_tzid(itime_tz);
          l_icaltimezone=icaltimezone_get_builtin_timezone(stz_loc);
          if (!l_icaltimezone) {
-            g_warning("Orage xfical_icalcomponent_archive_recurrent: builtin timezone %s not found, conversion failed.", stz_loc);
+            g_warning("xfical_icalcomponent_archive_recurrent: builtin timezone %s not found, conversion failed.", stz_loc);
         }
         sdate = icaltime_convert_to_zone(sdate, l_icaltimezone);
     }
@@ -1949,7 +2125,7 @@ void xfical_icalcomponent_archive_recurrent(icalcomponent *e
              etz_loc = (char *) icalparameter_get_tzid(itime_tz);
              l_icaltimezone=icaltimezone_get_builtin_timezone(etz_loc);
              if (!l_icaltimezone) {
-                g_warning("Orage xfical_icalcomponent_archive_recurrent: builtin timezone %s not found, conversion failed.", etz_loc);
+                g_warning("xfical_icalcomponent_archive_recurrent: builtin timezone %s not found, conversion failed.", etz_loc);
             }
             edate = icaltime_convert_to_zone(edate, l_icaltimezone);
         }
@@ -2028,11 +2204,11 @@ gboolean xfical_keep_tidy(void)
     char *uid;
 
     if (lookback == 0) {
-        g_message("Orage archiving not enabled. Exiting");
+        g_message("Orage **: Archiving not enabled. Exiting");
         return(TRUE);
     }
     if (!xfical_file_open() || !xfical_archive_open()) {
-        g_message("Orage xfical_keep_tidy: file open error");
+        g_warning("xfical_keep_tidy: file open error");
         return(FALSE);
     }
     threshold = orage_localtime();
@@ -2047,9 +2223,9 @@ gboolean xfical_keep_tidy(void)
     }
     threshold->tm_mon += 1;
 
-    g_message("Orage archiving threshold: %d month(s)", lookback);
-    g_message("Orage archiving before: %04d-%02d-%02d", threshold->tm_year
-            , threshold->tm_mon, threshold->tm_mday);
+    g_message("Orage **: Archiving threshold: %d month(s)\n\tArchiving before: %04d-%02d-%02d"
+            , lookback
+            , threshold->tm_year , threshold->tm_mon, threshold->tm_mday);
 
     /* Check appointment file for items older than the threshold */
     /* Note: remove moves the "c" pointer to next item, so we need to store it
@@ -2073,8 +2249,8 @@ gboolean xfical_keep_tidy(void)
             < ((threshold->tm_year)*12 + (threshold->tm_mon))) {
     /* Read from appointment files and copy into archive file */
             p = icalcomponent_get_first_property(c, ICAL_RRULE_PROPERTY);
-            g_message("Archiving uid: %s (%s)", uid, (p) ? "recur" : "normal");
-            g_message("\tEnd year: %04d, month: %02d, day: %02d"
+            g_message("Orage **: Archiving uid: %s (%s)\n\tEnd year: %04d, month: %02d, day: %02d"
+                    , uid, (p) ? "recur" : "normal"
                     , edate.year, edate.month, edate.day);
             if (p)  /*  it is recurrent event */
                 xfical_icalcomponent_archive_recurrent(c, threshold, uid);
@@ -2087,6 +2263,6 @@ gboolean xfical_keep_tidy(void)
     icalset_commit(afical);
     icalset_mark(fical);
     icalset_commit(fical);
-    g_message("Orage archiving threshold: %d month(s) done", lookback);
+    g_message("Orage **: Archiving done");
     return(TRUE);
 }
